@@ -89,8 +89,8 @@ int gnutls_random_art(gnutls_random_art_t type,
  * a server sends a prime with less bits than that
  * %GNUTLS_E_DH_PRIME_UNACCEPTABLE will be returned by the handshake.
  *
- * Note that values lower than 512 bits may allow decryption of the
- * exchanged data.
+ * Note that this function will warn via the audit log for value that
+ * are believed to be weak.
  *
  * The function has no effect in server side.
  * 
@@ -103,7 +103,7 @@ int gnutls_random_art(gnutls_random_art_t type,
  **/
 void gnutls_dh_set_prime_bits(gnutls_session_t session, unsigned int bits)
 {
-	if (bits <= gnutls_sec_param_to_pk_bits(GNUTLS_PK_DH, GNUTLS_SEC_PARAM_VERY_WEAK)
+	if (bits < gnutls_sec_param_to_pk_bits(GNUTLS_PK_DH, GNUTLS_SEC_PARAM_WEAK)
 		&& bits != 0)
 		_gnutls_audit_log(session,
 				  "Note that the security level of the Diffie-Hellman key exchange has been lowered to %u bits and this may allow decryption of the session data\n",
@@ -124,6 +124,9 @@ void gnutls_dh_set_prime_bits(gnutls_session_t session, unsigned int bits)
  * anonymous and ephemeral Diffie-Hellman.  The output parameters must
  * be freed with gnutls_free().
  *
+ * Note, that the prime and generator are exported as non-negative
+ * integers and may include a leading zero byte.
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
  *   an error code is returned.
  **/
@@ -189,6 +192,9 @@ gnutls_dh_get_group(gnutls_session_t session,
  * anonymous and ephemeral Diffie-Hellman.  The output parameters must
  * be freed with gnutls_free().
  *
+ * Note, that public key is exported as non-negative
+ * integer and may include a leading zero byte.
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
  *   an error code is returned.
  **/
@@ -356,6 +362,9 @@ int gnutls_dh_get_prime_bits(gnutls_session_t session)
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	if(dh->prime.size == 0)
+		return 0;
+
 	return mpi_buf2bits(&dh->prime);
 }
 
@@ -459,7 +468,7 @@ const gnutls_datum_t *gnutls_certificate_get_ours(gnutls_session_t session)
 
 	cred = (gnutls_certificate_credentials_t)
 	    _gnutls_get_cred(session, GNUTLS_CRD_CERTIFICATE);
-	if (cred == NULL || cred->certs == NULL) {
+	if (cred == NULL) {
 		gnutls_assert();
 		return NULL;
 	}
@@ -613,6 +622,24 @@ gnutls_certificate_set_params_function(gnutls_certificate_credentials_t
 	res->params_func = func;
 }
 
+/**
+ * gnutls_certificate_set_flags:
+ * @res: is a gnutls_certificate_credentials_t type
+ * @flags: are the flags of #gnutls_certificate_flags type
+ *
+ * This function will set flags to tweak the operation of
+ * the credentials structure. See the #gnutls_certificate_flags enumerations
+ * for more information on the available flags. 
+ *
+ * Since: 3.4.7
+ **/
+void
+gnutls_certificate_set_flags(gnutls_certificate_credentials_t res,
+			     unsigned int flags)
+{
+	res->flags = flags;
+}
+
 /**
  * gnutls_certificate_set_verify_flags:
  * @res: is a gnutls_certificate_credentials_t type
